#include "genesis.h"
#include "anim.h"
#include "sprite.h"
#include "unpack.h"
#include "debug.h"

/********* TO DO **********
 *
 * - Anim UpdateOne (behaviors)
 * - Anim Kill
 *
 **************************/

struct AnimatedSprite	Anim[MAX_ANIMS];

/******* ANIM_FINDFREE ******************
 * find the first unactive animSprite
 ****************************************/
static uchar Anim_FindFree( uchar searchFrom , uchar searchTo)
{
	uchar i;
	
	for(i=searchFrom; i<searchTo; i++)
	{
		if (Anim[i].active == FALSE)
		{
				return (i);
		}
	}
	
	return (0xFF);
}

/***** ANIM_UPDATEON **********
 * update an animSprite (frame, ticks)
 ****************************/
static void Anim_UpdateOne( uchar idxSpr )
{
	ushort posx, posy, spIdx;
	struct str_Frame *frame;
		
	if (Anim[idxSpr].active == FALSE)	return;
	
	frame = (struct str_Frame *) Anim[idxSpr].AnimLib->Frames[ Anim[idxSpr].curFrame ];
	Anim[idxSpr].curTick++;

	if ( Anim[idxSpr].curTick >= frame->Timer )
	{
		Anim[idxSpr].curFrame = frame->Next;
		Anim[idxSpr].curTick = 0;		
	}		
}

/********* ANIM_RAZ ********
 * free all animSprites
 ***************************/
void Anim_Init( )
{
	uchar i;
	
	for(i=0; i<MAX_ANIMS; i++)
			Anim_Delete(i);						
}

/******* ANIM_LOAD ************
 * load the tiles of an animLib
 ******************************/
ushort Anim_Load( struct str_Anim *AnimLib, ushort idxTile)
{
	ulong addr;
	
	if (AnimLib->compressSize > 0)
	{
		addr = idxTile*32;
		unpack(AnimLib->sprite_data, GFX_WRITE_ADDR(addr), AnimLib->compressSize);
	}
	else
	{
		dma_vram_copy( (uint) AnimLib->sprite_data, 32*idxTile, AnimLib->nbTiles*32 );
	}

	return ( idxTile + AnimLib->nbTiles );
}

/****** ANIM_NEW *********
 * init a new animSprite
 *************************/
uchar Anim_New( uchar IdCode, struct str_Anim *AnimLib, ushort firstTile, ushort posX, ushort posY, uchar pal)
{
	uchar i;
	i = Anim_FindFree( 0, MAX_ANIMS );
		
	if (i == 0xFF)	return (0xFF);
	
	Anim[i].active = TRUE;
	Anim[i].IDCode = IdCode;
	Anim[i].bhvIdx = 0;
	Anim[i].idxTile = firstTile;
	Anim[i].curFrame = 0;
	Anim[i].curTick = 0;
	Anim[i].x = posX;
	Anim[i].y = posY;
	Anim[i].palIdx = pal;
	Anim[i].flip = 0;
	Anim[i].data = 0;//data;
	Anim[i].AnimLib=AnimLib;
	
	return (i);
}

/********ANIM_DELETE ******
 * inactive an animSprite
 **************************/
void Anim_Delete( uchar idxSpr )
{
	Anim[idxSpr].active = FALSE;
}

/***** ANIM_UPADE **********
 * update all the animSprite
 * and draw them
 ****************************/
void Anim_Update(  )
{
	Anim_Update1( );
	Anim_Update2( );
}

void Anim_Update1(  )
{
	uchar i, j;	

	if ( PauseState )		return;
	
	for(i=0; i<MAX_ANIMS; i++)
	{		
//		Script_UpdateOne( i ); // update position and activity
		Anim_UpdateOne( i ); // update anim
	}
}

void Anim_Update2(  )
{
	uchar i;	

	if ( PauseState )		return;

	Sprite_Init( FALSE );

	for(i=0; i<MAX_ANIMS; i++)
	{
		Sprite_Add( i ); // add sprite
	}
		
	Sprite_Show( ); // show them
}
	
/***** ANIM_JUMPTO ********
 * jump to a specific frame
 **************************/
BOOL	Anim_JumpTo( uchar idxSpr, uchar toFrame)
{
	if ( Anim[idxSpr].active == FALSE) return FALSE;
	
	if ( toFrame >= Anim[idxSpr].AnimLib->nbFrames)	return FALSE;
	
	
	Anim[idxSpr].curFrame = toFrame;	
	Anim[idxSpr].curTick = 0;
	
	return TRUE;
}

/**/