#include "genesis.h"
#include "control.h"
#include "game.h"
#include "level.h"
#include "tetris.h"

struct Player P1;
struct Player P2;


uchar GameMode;

/******* PRIVATE ********/
static void Game_Score( struct Player *P )
{
	ushort tile, score;
	char i;
	
	tile = 0x8000 +17;
	score = P->nScore;

	for(i=5; i>=0; i--)
	{
		Gen_ShowUnique(  16+i, 12, tile + (score%10), APLAN);
		score /= 10;
	}
	
	P->nLevel = MIN( 9, (P->nScore/100));
}

static void Game_Update_Player(struct Player *P, struct Joy *J)
{
	BOOL	bEndPiece;
	
	bEndPiece = FALSE;
	
	
	if (P->bTetris)
	{
	/*
		Tetris_Update(P);				
		return;
			*/
		P->bTetris = FALSE;
		Game_Score( P );
	}
	
	if (P->strPiece.nIDShape == SHAPE_NONE)
	{
		Piece_Init( &(P->strPiece), P->nID_Next);
		P->nID_Next=Piece_Random( );
		P->nDelay = (10 - P->nLevel) * 5 ;
		return;
	}
	
		
	if (J->key&JOY_DOWN)
	{
		P->strPiece.bDropping = 1;
	}

	if (J->key&JOY_LEFT)
	{
		Piece_Left( &(P->strPiece), (uchar *)P->aBoard );
	}
	else if (J->key&JOY_RIGHT)
	{
		Piece_Right( &(P->strPiece), (uchar *)P->aBoard );
	}
	
	if (J->key&BUTTON_A)
	{
		Piece_RotateR( &(P->strPiece), (uchar *)P->aBoard );
	}
	else if (J->key&BUTTON_B)
	{
		Piece_RotateL( &(P->strPiece), (uchar *)P->aBoard );
	}

	
	if 	(P->strPiece.bDropping || P->nDelay == 0)
	{
		bEndPiece = Piece_Down( &(P->strPiece), (uchar *)P->aBoard );
		P->nDelay = (10 - P->nLevel) * 5 ;
	}
	else
	{
		P->nDelay--;
	}
	
	Piece_Update( &(P->strPiece) );	
	if  (bEndPiece)
	{
		// update board
		Board_Add( (uchar *)P->aBoard, &(P->strPiece));
		
		// check Tetris
		bEndPiece = Tetris_Check( P );
		if ( (bEndPiece == FALSE) && (P->strPiece.nY == 0) )
		{
			ResetState = 1; // A MODIFIER
			return;
		}
				
		// draw board
		Board_Draw( (uchar *)P->aBoard, 1, 2); // A MODIFIER
		
		// init reset
		P->strPiece.nIDShape = SHAPE_NONE;
	}
}


/************** PUBLIC *************/
void AlertNumber(ulong nVal)
{
	uchar string[8];
	
	string[7] = 0;
	string[6] = (nVal&0x0F)+48;
	if (string[6]>57)	string[6]+=7;
	nVal >>=4;
	string[5] = (nVal&0x0F)+48;
	if (string[5]>57)	string[5]+=7;
	nVal >>=4;
	string[4] = (nVal&0x0F)+48;
	if (string[4]>57)	string[4]+=7;
	nVal >>=4;
	string[3] = (nVal&0x0F)+48;
	if (string[3]>57)	string[3]+=7;
	nVal >>=4;
	string[2] = (nVal&0x0F)+48;
	if (string[2]>57)	string[2]+=7;
	nVal >>=4;
	string[1] = (nVal&0x0F)+48;
	if (string[1]>57)	string[1]+=7;
	nVal >>=4;
	string[0] = (nVal&0x0F)+48;
	if (string[0]>57)	string[0]+=7;
	
	AlertEmu( string );
}



void Game_Init( uchar nMode )
{
	uchar i,j;
	
	GameMode = nMode;
	
	// init P1
	P1.nType = HUMAN_PLAYER;
	P1.nScore = 0;
	P1.nLevel = 0;
	
	for(i=0; i<20; i++)
	{	
		P1.aBoard[i][0] = 1;
		for (j=1; j < 11; j++)
			P1.aBoard[i][j] = 0;
		P1.aBoard[i][11] = 1;
	}

	
	P1.nDelay=0; ///// A REVOIR
	P1.nID_Next=Piece_Random( );
	P1.bTetris=0;
	for(i=0;i<4;i++)
		P1.aTetrisLine[i] = 0;
	
	Piece_Reset(&P1.strPiece);
	P1.nMin = 4;
	P1.nMax = 14;
	
	
	// init P2
	P2.nType = HUMAN_PLAYER;
	P2.nScore = 0;
	P2.nLevel = 0;
	
	for(i=0; i<20; i++)
	{	
		P2.aBoard[i][0] = 1;
		for (j=1; j < 11; j++)
			P2.aBoard[i][j] = 0;
		P2.aBoard[i][11] = 1;
	}
	
	P2.nDelay=0; ///// A REVOIR
	P2.nID_Next=Piece_Random( );
	P2.bTetris=0;
	for(i=0;i<4;i++)
		P2.aTetrisLine[i] = 0;
	
	Piece_Reset(&P2.strPiece);
	P2.nMin = 18;
	P2.nMax = 28;
	
	
	switch(nMode)
	{
		case 0: 	//human vs none
			P1.nMin = 11;
			P1.nMax = 21;
			P2.nType = NO_PLAYER;
			
			Game_Score( &P1 );
			Board_Draw( P1.aBoard, 1, 2);
			break;
			
		case 1:  //human vs human
			break;
		
		case 2:  //human vs CPU
			P2.nType = CPU_PLAYER;
			break;
	}		
}

void Game_Update( )
{
	if (GameMode == 0)
	{
		Game_Update_Player(&P1, &Joypad1);		
	}
	else
	{
		Game_Update_Player(&P1, &Joypad1);
		Game_Update_Player(&P2, &Joypad2);
	}
}



