#include "genesis.h"
#include "piece.h"
#include "anim.h"
#include "level.h"


//ushort piece_seed;

uchar Piece_Random( )
{
/*
	if (piece_seed==0)	piece_seed = (vtimer & 0xFFFF);
	
	// "linear congruential" method 
	piece_seed = (109 * piece_seed + 853) % 4096; 


	return (piece_seed%(SHAPE_T+1));
*/
	
	return ( vtimer % (SHAPE_T+1));
}


BOOL Piece_Intersect( struct Piece *P, uchar *Array2 )
{
	uchar posX, posY;
	
	for (posY=0; posY<4; posY++)
	{
		for (posX=0; posX<4; posX++)
		{
				// something to check ?
				if ( P->aMask[posY][posX] )
				{
					// if piece outside zone, 
					if ( (posY+ P->nY) >= 20 ) return TRUE;
	
					// if a block is present at the same place, collide!
					if ( *(Array2 + 12*(posY + P->nY) + (posX+ P->nX +1)) )	return TRUE;
				}
		}		
	}
	
	return FALSE;
}


void Piece_RemapMask( struct Piece *P )
{
	uchar posX, posY,i;
	struct str_Frame *frame;
	ushort tmp;


	frame = (struct str_Frame *) Anim[P->nAnimIdx].AnimLib->Frames[ P->nCurFrame ];

	// RAZ all
	for (posY=0; posY<4; posY++)
	{
		for (posX=0; posX<4; posX++)
		{
			P->aMask[posY][posX] = 0;
		}		
	}	
	
	
	// remap
	for( i=0; i < frame->nbSprites; i++)
	{	
		P->aMask[ frame->Sprites[i].Y / 8 ][ frame->Sprites[i].X / 8 ] = 1;
	}
	
	// DEBUG
/*	
	AlertEmu("Mask");
	for (posY=0; posY<4; posY++)
	{
		tmp = 0;
		for (posX=0; posX<4; posX++)
		{
			if (P->aMask[posY][posX]) tmp |= ( 1 << 4*(3-posX));
		}
		AlertNumber(tmp);
	}	
	*/
}



/************ PUBLIC *************/
void Piece_Reset( struct Piece *P)
{
	if (P->nAnimIdx != 0xFF)	Anim_Delete( P->nAnimIdx );
	P->nCurFrame = 0;
	P->nX = 0;
	P->nY = 0;
	P->nIDShape = SHAPE_NONE;
	P->bDropping = 0;
}

void Piece_Init( struct Piece *P, uchar IDShape)
{
	Piece_Reset(P);
	P->nX = 3;
	P->nY = 0;
	P->nIDShape = IDShape;	
	P->nAnimIdx = Anim_New( 00, &pieces_anim, curLevel.data, 128 + (P->nX + 1)*8, 128 + (P->nY+ 2)*8, PAL2);
	
	switch(IDShape)
	{
		case SHAPE_NONE:
			P->nMinFrame = 0;
			P->nMaxFrame = 0;
			break;
	
		case SHAPE_O:
			P->nMinFrame = 0;
			P->nMaxFrame = 0;
			break;
			
		case SHAPE_I:
			P->nMinFrame = 1;
			P->nMaxFrame = 2;
			break;

		case SHAPE_S:
			P->nMinFrame = 3;
			P->nMaxFrame = 4;
			break;

		case SHAPE_Z:
			P->nMinFrame = 5;
			P->nMaxFrame = 6;
			break;

		case SHAPE_L:
			P->nMinFrame = 7;
			P->nMaxFrame = 10;
			break;

		case SHAPE_J:
			P->nMinFrame = 11;
			P->nMaxFrame = 14;
			break;

		case SHAPE_T:
			P->nMinFrame = 15;
			P->nMaxFrame = 18;
			break;
	}
	
	P->nCurFrame = P->nMinFrame;
	Piece_RemapMask( P );
	Anim_JumpTo( P->nAnimIdx, P->nCurFrame );
	
}

void Piece_Update( struct Piece *P )
{
	Anim_Update(  );
}


////////// REVOIR LES BOOL communs
BOOL Piece_Down( struct Piece *P, uchar *RefArray )
{
	P->nY++;
	Piece_RemapMask( P );

	if ( Piece_Intersect( P, RefArray ) )
	{
		P->nY--;
		Piece_RemapMask( P );
		return TRUE;
	}

	Anim[P->nAnimIdx].y += 8;
	
	return FALSE;
}

BOOL Piece_Left( struct Piece *P, uchar *RefArray )
{
	P->nX--;
	Piece_RemapMask( P );

	if ( Piece_Intersect( P, RefArray ) )
	{
		P->nX++;
		Piece_RemapMask( P );
		return FALSE;
	}
		
	Anim[P->nAnimIdx].x -= 8;
	return FALSE;
}

BOOL Piece_Right( struct Piece *P, uchar *RefArray )
{
	P->nX++;
	Piece_RemapMask( P );
	if ( Piece_Intersect( P, RefArray ) )
	{
		P->nX--;
		Piece_RemapMask( P );
		return FALSE;
	}
		
	Anim[P->nAnimIdx].x += 8;
	return FALSE;
}

BOOL Piece_RotateL( struct Piece *P, uchar *RefArray )
{
	if ( P->nCurFrame == P->nMinFrame)
		P->nCurFrame = P->nMaxFrame;
	else
		P->nCurFrame--;

	Piece_RemapMask( P );		
	if ( Piece_Intersect( P, RefArray ) )
	{
		if ( P->nCurFrame == P->nMaxFrame)
			P->nCurFrame = P->nMinFrame;
		else
			P->nCurFrame++;
		Piece_RemapMask( P );
		return FALSE;
	}
		
	Anim_JumpTo( P->nAnimIdx, P->nCurFrame );
	
	return FALSE;
}

BOOL Piece_RotateR( struct Piece *P, uchar *RefArray )
{
	if ( P->nCurFrame == P->nMaxFrame)
		P->nCurFrame = P->nMinFrame;
	else
		P->nCurFrame++;
	
	Piece_RemapMask( P );
	if ( Piece_Intersect( P, RefArray ) )
	{
		if ( P->nCurFrame == P->nMinFrame)
			P->nCurFrame = P->nMaxFrame;
		else
			P->nCurFrame--;
		Piece_RemapMask( P );
		return FALSE;
	}
	
	Anim_JumpTo( P->nAnimIdx, P->nCurFrame );
	
	return FALSE;
}
